@@ -1,4 +1,5 @@
 /*
+ *  Copyright (C) 2011-2012 Christian Beier <dontmind@freeshell.org>
  *  Copyright (C) 1999 AT&T Laboratories Cambridge.  All Rights Reserved.
  *
  *  This is free software; you can redistribute it and/or modify
@@ -21,24 +22,35 @@
  * sockets.c - functions to deal with sockets.
  */
 
+#ifdef __STRICT_ANSI__
+#define _BSD_SOURCE
+#endif
 #include <unistd.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <assert.h>
 #include <rfb/rfbclient.h>
 #ifdef WIN32
+#undef SOCKET
 #include <winsock2.h>
 #define EWOULDBLOCK WSAEWOULDBLOCK
 #define close closesocket
 #define read(sock,buf,len) recv(sock,buf,len,0)
 #define write(sock,buf,len) send(sock,buf,len,0)
+#define socklen_t int
+#ifdef LIBVNCSERVER_HAVE_WS2TCPIP_H
+#undef socklen_t
+#include <ws2tcpip.h>
+#endif
 #else
 #include <sys/socket.h>
 #include <netinet/in.h>
+#include <sys/un.h>
 #include <netinet/tcp.h>
 #include <arpa/inet.h>
 #include <netdb.h>
 #endif
+#include "tls.h"
 
 void PrintInHex(char *buf, int len);
 
@@ -88,12 +100,11 @@ ReadFromRFBServer(rfbClient* client, char *out, unsigned int n)
 	  diff.tv_sec--;
 	  diff.tv_usec+=1000000;
         }
-        sleep (diff.tv_sec);
 #ifndef __MINGW32__
-	/* FIXME */
+        sleep (diff.tv_sec);
         usleep (diff.tv_usec);
 #else
-	rfbClientErr("usleep on MinGW32 NOT IMPLEMENTED\n");
+	Sleep (diff.tv_sec * 1000 + diff.tv_usec/1000);
 #endif
       }
 
@@ -124,7 +135,12 @@ ReadFromRFBServer(rfbClient* client, char *out, unsigned int n)
   if (n <= RFB_BUF_SIZE) {
 
     while (client->buffered < n) {
-      int i = read(client->sock, client->buf + client->buffered, RFB_BUF_SIZE - client->buffered);
+      int i;
+      if (client->tlsSession) {
+        i = ReadFromTLS(client, client->buf + client->buffered, RFB_BUF_SIZE - client->buffered);
+      } else {
+        i = read(client->sock, client->buf + client->buffered, RFB_BUF_SIZE - client->buffered);
+      }
       if (i <= 0) {
 	if (i < 0) {
 #ifdef WIN32
@@ -134,6 +150,7 @@ ReadFromRFBServer(rfbClient* client, char *out, unsigned int n)
 	    /* TODO:
 	       ProcessXtEvents();
 	    */
+	    WaitForMessage(client, 100000);
 	    i = 0;
 	  } else {
 	    rfbClientErr("read (%d: %s)\n",errno,strerror(errno));
@@ -156,7 +173,13 @@ ReadFromRFBServer(rfbClient* client, char *out, unsigned int n)
   } else {
 
     while (n > 0) {
-      int i = read(client->sock, out, n);
+      int i;
+      if (client->tlsSession) {
+        i = ReadFromTLS(client, out, n);
+      } else {
+        i = read(client->sock, out, n);
+      }
+
       if (i <= 0) {
 	if (i < 0) {
 #ifdef WIN32
@@ -166,6 +189,7 @@ ReadFromRFBServer(rfbClient* client, char *out, unsigned int n)
 	    /* TODO:
 	       ProcessXtEvents();
 	    */
+	    WaitForMessage(client, 100000);
 	    i = 0;
 	  } else {
 	    rfbClientErr("read (%s)\n",strerror(errno));
@@ -210,11 +234,26 @@ WriteToRFBServer(rfbClient* client, char *buf, int n)
   if (client->serverPort==-1)
     return TRUE; /* vncrec playing */
 
+  if (client->tlsSession) {
+    /* WriteToTLS() will guarantee either everything is written, or error/eof returns */
+    i = WriteToTLS(client, buf, n);
+    if (i <= 0) return FALSE;
+
+    return TRUE;
+  }
+
   while (i < n) {
     j = write(client->sock, buf + i, (n - i));
     if (j <= 0) {
       if (j < 0) {
-	if (errno == EWOULDBLOCK || errno == EAGAIN) {
+#ifdef WIN32
+	 errno=WSAGetLastError();
+#endif
+	if (errno == EWOULDBLOCK ||
+#ifdef LIBVNCSERVER_ENOENT_WORKAROUND
+		errno == ENOENT ||
+#endif
+		errno == EAGAIN) {
 	  FD_ZERO(&fds);
 	  FD_SET(client->sock,&fds);
 
@@ -238,6 +277,23 @@ WriteToRFBServer(rfbClient* client, char *buf, int n)
 }
 
 
+
+static int initSockets() {
+#ifdef WIN32
+  WSADATA trash;
+  static rfbBool WSAinitted=FALSE;
+  if(!WSAinitted) {
+    int i=WSAStartup(MAKEWORD(2,0),&trash);
+    if(i!=0) {
+      rfbClientErr("Couldn't init Windows Sockets\n");
+      return 0;
+    }
+    WSAinitted=TRUE;
+  }
+#endif
+  return 1;
+}
+
 /*
  * ConnectToTcpAddr connects to the given TCP port.
  */
@@ -249,18 +305,8 @@ ConnectClientToTcpAddr(unsigned int host, int port)
   struct sockaddr_in addr;
   int one = 1;
 
-#ifdef WIN32
-  WSADATA trash;
-  static rfbBool WSAinitted=FALSE;
-  if(!WSAinitted) {
-    WSAinitted=TRUE;
-    int i=WSAStartup(MAKEWORD(2,0),&trash);
-    if(i!=0) {
-      rfbClientErr("Couldn't init Windows Sockets\n");
-      return -1;
-    }
-  }
-#endif
+  if (!initSockets())
+	  return -1;
 
   addr.sin_family = AF_INET;
   addr.sin_port = htons(port);
@@ -291,6 +337,96 @@ ConnectClientToTcpAddr(unsigned int host, int port)
   return sock;
 }
 
+int
+ConnectClientToTcpAddr6(const char *hostname, int port)
+{
+#ifdef LIBVNCSERVER_IPv6
+  int sock;
+  int n;
+  struct addrinfo hints, *res, *ressave;
+  char port_s[10];
+  int one = 1;
+
+  if (!initSockets())
+	  return -1;
+
+  snprintf(port_s, 10, "%d", port);
+  memset(&hints, 0, sizeof(struct addrinfo));
+  hints.ai_family = AF_UNSPEC;
+  hints.ai_socktype = SOCK_STREAM;
+  if ((n = getaddrinfo(hostname, port_s, &hints, &res)))
+  {
+    rfbClientErr("ConnectClientToTcpAddr6: getaddrinfo (%s)\n", gai_strerror(n));
+    return -1;
+  }
+
+  ressave = res;
+  sock = -1;
+  while (res)
+  {
+    sock = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
+    if (sock >= 0)
+    {
+      if (connect(sock, res->ai_addr, res->ai_addrlen) == 0)
+        break;
+      close(sock);
+      sock = -1;
+    }
+    res = res->ai_next;
+  }
+  freeaddrinfo(ressave);
+
+  if (sock == -1)
+  {
+    rfbClientErr("ConnectClientToTcpAddr6: connect\n");
+    return -1;
+  }
+
+  if (setsockopt(sock, IPPROTO_TCP, TCP_NODELAY,
+		 (char *)&one, sizeof(one)) < 0) {
+    rfbClientErr("ConnectToTcpAddr: setsockopt\n");
+    close(sock);
+    return -1;
+  }
+
+  return sock;
+
+#else
+
+  rfbClientErr("ConnectClientToTcpAddr6: IPv6 disabled\n");
+  return -1;
+
+#endif
+}
+
+int
+ConnectClientToUnixSock(const char *sockFile)
+{
+#ifdef WIN32
+  rfbClientErr("Windows doesn't support UNIX sockets\n");
+  return -1;
+#else
+  int sock;
+  struct sockaddr_un addr;
+  addr.sun_family = AF_UNIX;
+  strcpy(addr.sun_path, sockFile);
+
+  sock = socket(AF_UNIX, SOCK_STREAM, 0);
+  if (sock < 0) {
+    rfbClientErr("ConnectToUnixSock: socket (%s)\n",strerror(errno));
+    return -1;
+  }
+
+  if (connect(sock, (struct sockaddr *)&addr, sizeof(addr.sun_family) + strlen(addr.sun_path)) < 0) {
+    rfbClientErr("ConnectToUnixSock: connect\n");
+    close(sock);
+    return -1;
+  }
+
+  return sock;
+#endif
+}
+
 
 
 /*
@@ -305,7 +441,10 @@ FindFreeTcpPort(void)
   struct sockaddr_in addr;
 
   addr.sin_family = AF_INET;
-  addr.sin_addr.s_addr = INADDR_ANY;
+  addr.sin_addr.s_addr = htonl(INADDR_ANY);
+
+  if (!initSockets())
+    return -1;
 
   sock = socket(AF_INET, SOCK_STREAM, 0);
   if (sock < 0) {
@@ -332,14 +471,35 @@ FindFreeTcpPort(void)
 
 int
 ListenAtTcpPort(int port)
+{
+  return ListenAtTcpPortAndAddress(port, NULL);
+}
+
+
+
+/*
+ * ListenAtTcpPortAndAddress starts listening at the given TCP port on
+ * the given IP address
+ */
+
+int
+ListenAtTcpPortAndAddress(int port, const char *address)
 {
   int sock;
-  struct sockaddr_in addr;
   int one = 1;
+#ifndef LIBVNCSERVER_IPv6
+  struct sockaddr_in addr;
 
   addr.sin_family = AF_INET;
   addr.sin_port = htons(port);
-  addr.sin_addr.s_addr = INADDR_ANY;
+  if (address) {
+    addr.sin_addr.s_addr = inet_addr(address);
+  } else {
+    addr.sin_addr.s_addr = htonl(INADDR_ANY);
+  }
+
+  if (!initSockets())
+    return -1;
 
   sock = socket(AF_INET, SOCK_STREAM, 0);
   if (sock < 0) {
@@ -360,6 +520,66 @@ ListenAtTcpPort(int port)
     return -1;
   }
 
+#else
+  int rv;
+  struct addrinfo hints, *servinfo, *p;
+  char port_str[8];
+
+  snprintf(port_str, 8, "%d", port);
+
+  memset(&hints, 0, sizeof(hints));
+  hints.ai_family = AF_UNSPEC;
+  hints.ai_socktype = SOCK_STREAM;
+  hints.ai_flags = AI_PASSIVE; /* fill in wildcard address if address == NULL */
+
+  if (!initSockets())
+    return -1;
+
+  if ((rv = getaddrinfo(address, port_str, &hints, &servinfo)) != 0) {
+    rfbClientErr("ListenAtTcpPortAndAddress: error in getaddrinfo: %s\n", gai_strerror(rv));
+    return -1;
+  }
+
+  /* loop through all the results and bind to the first we can */
+  for(p = servinfo; p != NULL; p = p->ai_next) {
+    if ((sock = socket(p->ai_family, p->ai_socktype, p->ai_protocol)) < 0) {
+      continue;
+    }
+
+#ifdef IPV6_V6ONLY
+    /* we have seperate IPv4 and IPv6 sockets since some OS's do not support dual binding */
+    if (p->ai_family == AF_INET6 && setsockopt(sock, IPPROTO_IPV6, IPV6_V6ONLY, (char *)&one, sizeof(one)) < 0) {
+      rfbClientErr("ListenAtTcpPortAndAddress: error in setsockopt IPV6_V6ONLY: %s\n", strerror(errno));
+      close(sock);
+      freeaddrinfo(servinfo);
+      return -1;
+    }
+#endif
+
+    if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(one)) < 0) {
+      rfbClientErr("ListenAtTcpPortAndAddress: error in setsockopt SO_REUSEADDR: %s\n", strerror(errno));
+      close(sock);
+      freeaddrinfo(servinfo);
+      return -1;
+    }
+
+    if (bind(sock, p->ai_addr, p->ai_addrlen) < 0) {
+      close(sock);
+      continue;
+    }
+
+    break;
+  }
+
+  if (p == NULL)  {
+    rfbClientErr("ListenAtTcpPortAndAddress: error in bind: %s\n", strerror(errno));
+    return -1;
+  }
+
+  /* all done with this structure now */
+  freeaddrinfo(servinfo);
+#endif
+
   if (listen(sock, 5) < 0) {
     rfbClientErr("ListenAtTcpPort: listen\n");
     close(sock);
@@ -379,7 +599,7 @@ AcceptTcpConnection(int listenSock)
 {
   int sock;
   struct sockaddr_in addr;
-  int addrlen = sizeof(addr);
+  socklen_t addrlen = sizeof(addr);
   int one = 1;
 
   sock = accept(listenSock, (struct sockaddr *) &addr, &addrlen);
@@ -406,18 +626,70 @@ AcceptTcpConnection(int listenSock)
 rfbBool
 SetNonBlocking(int sock)
 {
-#ifndef __MINGW32__
-  if (fcntl(sock, F_SETFL, O_NONBLOCK) < 0) {
-    rfbClientErr("AcceptTcpConnection: fcntl\n");
+#ifdef WIN32
+  unsigned long block=1;
+  if(ioctlsocket(sock, FIONBIO, &block) == SOCKET_ERROR) {
+    errno=WSAGetLastError();
+#else
+  int flags = fcntl(sock, F_GETFL);
+  if(flags < 0 || fcntl(sock, F_SETFL, flags | O_NONBLOCK) < 0) {
+#endif
+    rfbClientErr("Setting socket to non-blocking failed: %s\n",strerror(errno));
     return FALSE;
   }
-#else
-  rfbClientErr("O_NONBLOCK on MinGW32 NOT IMPLEMENTED\n");
-#endif
   return TRUE;
 }
 
 
+
+/*
+ * SetDSCP sets a socket's IP QoS parameters aka Differentiated Services Code Point field
+ */
+
+rfbBool
+SetDSCP(int sock, int dscp)
+{
+#ifdef WIN32
+  rfbClientErr("Setting of QoS IP DSCP not implemented for Windows\n");
+  return TRUE;
+#else
+  int level, cmd;
+  struct sockaddr addr;
+  socklen_t addrlen = sizeof(addr);
+
+  if(getsockname(sock, &addr, &addrlen) != 0) {
+    rfbClientErr("Setting socket QoS failed while getting socket address: %s\n",strerror(errno));
+    return FALSE;
+  }
+
+  switch(addr.sa_family)
+    {
+#if defined LIBVNCSERVER_IPv6 && defined IPV6_TCLASS
+    case AF_INET6:
+      level = IPPROTO_IPV6;
+      cmd = IPV6_TCLASS;
+      break;
+#endif
+    case AF_INET:
+      level = IPPROTO_IP;
+      cmd = IP_TOS;
+      break;
+    default:
+      rfbClientErr("Setting socket QoS failed: Not bound to IP address");
+      return FALSE;
+    }
+
+  if(setsockopt(sock, level, cmd, (void*)&dscp, sizeof(dscp)) != 0) {
+    rfbClientErr("Setting socket QoS failed: %s\n", strerror(errno));
+    return FALSE;
+  }
+
+  return TRUE;
+#endif
+}
+
+
+
 /*
  * StringToIPAddr - convert a host string to an IP address.
  */
@@ -428,7 +700,7 @@ StringToIPAddr(const char *str, unsigned int *addr)
   struct hostent *hp;
 
   if (strcmp(str,"") == 0) {
-    *addr = 0; /* local */
+    *addr = htonl(INADDR_LOOPBACK); /* local */
     return TRUE;
   }
 
@@ -437,6 +709,9 @@ StringToIPAddr(const char *str, unsigned int *addr)
   if (*addr != -1)
     return TRUE;
 
+  if (!initSockets())
+	  return -1;
+
   hp = gethostbyname(str);
 
   if (hp) {
@@ -456,7 +731,7 @@ rfbBool
 SameMachine(int sock)
 {
   struct sockaddr_in peeraddr, myaddr;
-  int addrlen = sizeof(struct sockaddr_in);
+  socklen_t addrlen = sizeof(struct sockaddr_in);
 
   getpeername(sock, (struct sockaddr *)&peeraddr, &addrlen);
   getsockname(sock, (struct sockaddr *)&myaddr, &addrlen);
@@ -525,8 +800,12 @@ int WaitForMessage(rfbClient* client,unsigned int usecs)
   FD_SET(client->sock,&fds);
 
   num=select(client->sock+1, &fds, NULL, NULL, &timeout);
-  if(num<0)
+  if(num<0) {
+#ifdef WIN32
+    errno=WSAGetLastError();
+#endif
     rfbClientLog("Waiting for message failed: %d (%s)\n",errno,strerror(errno));
+  }
 
   return num;
 }
