@@ -19,6 +19,7 @@
  */
 
 /*
+ *  Copyright (C) 2011-2012 Christian Beier <dontmind@freeshell.org>
  *  Copyright (C) 2005 Rohit Kumar, Johannes E. Schindelin
  *  OSXvnc Copyright (C) 2001 Dan McGuirk <mcguirk@incompleteness.net>.
  *  Original Xvnc code Copyright (C) 1999 AT&T Laboratories Cambridge.  
@@ -62,6 +63,10 @@
 #include <unistd.h>
 #endif
 
+#ifdef LIBVNCSERVER_WITH_WEBSOCKETS
+#include "rfbssl.h"
+#endif
+
 #if defined(__linux__) && defined(NEED_TIMEVAL)
 struct timeval 
 {
@@ -133,6 +138,8 @@ rfbInitSockets(rfbScreenInfoPtr rfbScreen)
 
     if(rfbScreen->autoPort) {
         int i;
+        FD_ZERO(&(rfbScreen->allFds));
+
         rfbLog("Autoprobing TCP port \n");
         for (i = 5900; i < 6000; i++) {
             if ((rfbScreen->listenSock = rfbListenOnTCPPort(i, iface)) >= 0) {
@@ -146,22 +153,52 @@ rfbInitSockets(rfbScreenInfoPtr rfbScreen)
 	    return;
         }
 
-        rfbLog("Autoprobing selected port %d\n", rfbScreen->port);
-        FD_ZERO(&(rfbScreen->allFds));
+        rfbLog("Autoprobing selected TCP port %d\n", rfbScreen->port);
         FD_SET(rfbScreen->listenSock, &(rfbScreen->allFds));
         rfbScreen->maxFd = rfbScreen->listenSock;
+
+#ifdef LIBVNCSERVER_IPv6
+        rfbLog("Autoprobing TCP6 port \n");
+	for (i = 5900; i < 6000; i++) {
+            if ((rfbScreen->listen6Sock = rfbListenOnTCP6Port(i, rfbScreen->listen6Interface)) >= 0) {
+		rfbScreen->ipv6port = i;
+		break;
+	    }
+        }
+
+        if (i >= 6000) {
+	    rfbLogPerror("Failure autoprobing");
+	    return;
+        }
+
+        rfbLog("Autoprobing selected TCP6 port %d\n", rfbScreen->ipv6port);
+	FD_SET(rfbScreen->listen6Sock, &(rfbScreen->allFds));
+	rfbScreen->maxFd = max((int)rfbScreen->listen6Sock,rfbScreen->maxFd);
+#endif
     }
     else if(rfbScreen->port>0) {
-      rfbLog("Listening for VNC connections on TCP port %d\n", rfbScreen->port);
+      FD_ZERO(&(rfbScreen->allFds));
 
       if ((rfbScreen->listenSock = rfbListenOnTCPPort(rfbScreen->port, iface)) < 0) {
 	rfbLogPerror("ListenOnTCPPort");
 	return;
       }
-
-      FD_ZERO(&(rfbScreen->allFds));
+      rfbLog("Listening for VNC connections on TCP port %d\n", rfbScreen->port);  
+  
       FD_SET(rfbScreen->listenSock, &(rfbScreen->allFds));
       rfbScreen->maxFd = rfbScreen->listenSock;
+
+#ifdef LIBVNCSERVER_IPv6
+      if ((rfbScreen->listen6Sock = rfbListenOnTCP6Port(rfbScreen->ipv6port, rfbScreen->listen6Interface)) < 0) {
+	/* ListenOnTCP6Port has its own detailed error printout */
+	return;
+      }
+      rfbLog("Listening for VNC connections on TCP6 port %d\n", rfbScreen->ipv6port);  
+	
+      FD_SET(rfbScreen->listen6Sock, &(rfbScreen->allFds));
+      rfbScreen->maxFd = max((int)rfbScreen->listen6Sock,rfbScreen->maxFd);
+#endif
+
     }
 
     if (rfbScreen->udpPort != 0) {
@@ -171,6 +208,8 @@ rfbInitSockets(rfbScreenInfoPtr rfbScreen)
 	    rfbLogPerror("ListenOnUDPPort");
 	    return;
 	}
+	rfbLog("Listening for VNC connections on TCP port %d\n", rfbScreen->port);  
+
 	FD_SET(rfbScreen->udpSock, &(rfbScreen->allFds));
 	rfbScreen->maxFd = max((int)rfbScreen->udpSock,rfbScreen->maxFd);
     }
@@ -195,6 +234,12 @@ void rfbShutdownSockets(rfbScreenInfoPtr rfbScreen)
 	rfbScreen->listenSock=-1;
     }
 
+    if(rfbScreen->listen6Sock>-1) {
+	closesocket(rfbScreen->listen6Sock);
+	FD_CLR(rfbScreen->listen6Sock,&rfbScreen->allFds);
+	rfbScreen->listen6Sock=-1;
+    }
+
     if(rfbScreen->udpSock>-1) {
 	closesocket(rfbScreen->udpSock);
 	FD_CLR(rfbScreen->udpSock,&rfbScreen->allFds);
@@ -218,8 +263,6 @@ rfbCheckFds(rfbScreenInfoPtr rfbScreen,long usec)
     struct sockaddr_in addr;
     socklen_t addrlen = sizeof(addr);
     char buf[6];
-    const int one = 1;
-    int sock;
     rfbClientIteratorPtr i;
     rfbClientPtr cl;
     int result = 0;
@@ -260,43 +303,24 @@ rfbCheckFds(rfbScreenInfoPtr rfbScreen,long usec)
 
 	if (rfbScreen->listenSock != -1 && FD_ISSET(rfbScreen->listenSock, &fds)) {
 
-	    if ((sock = accept(rfbScreen->listenSock,
-			    (struct sockaddr *)&addr, &addrlen)) < 0) {
-		rfbLogPerror("rfbCheckFds: accept");
-		return -1;
-	    }
-
-            if(!rfbSetNonBlocking(sock)) {
-	        closesocket(sock);
-		return -1;
-	    }
-
-	    if (setsockopt(sock, IPPROTO_TCP, TCP_NODELAY,
-			(char *)&one, sizeof(one)) < 0) {
-		rfbLogPerror("rfbCheckFds: setsockopt");
-		closesocket(sock);
-		return -1;
-	    }
-
-#ifdef USE_LIBWRAP
-	    if(!hosts_ctl("vnc",STRING_UNKNOWN,inet_ntoa(addr.sin_addr),
-			STRING_UNKNOWN)) {
-		rfbLog("Rejected connection from client %s\n",
-			inet_ntoa(addr.sin_addr));
-		closesocket(sock);
-		return -1;
-	    }
-#endif
-
-	    rfbLog("Got connection from client %s\n", inet_ntoa(addr.sin_addr));
-
-	    rfbNewClient(rfbScreen,sock);
+	    if (!rfbProcessNewConnection(rfbScreen))
+                return -1;
 
 	    FD_CLR(rfbScreen->listenSock, &fds);
 	    if (--nfds == 0)
 		return result;
 	}
 
+	if (rfbScreen->listen6Sock != -1 && FD_ISSET(rfbScreen->listen6Sock, &fds)) {
+
+	    if (!rfbProcessNewConnection(rfbScreen))
+                return -1;
+
+	    FD_CLR(rfbScreen->listen6Sock, &fds);
+	    if (--nfds == 0)
+		return result;
+	}
+
 	if ((rfbScreen->udpSock != -1) && FD_ISSET(rfbScreen->udpSock, &fds)) {
 	    if(!rfbScreen->udpClient)
 		rfbNewUDPClient(rfbScreen);
@@ -352,6 +376,80 @@ rfbCheckFds(rfbScreenInfoPtr rfbScreen,long usec)
     return result;
 }
 
+rfbBool
+rfbProcessNewConnection(rfbScreenInfoPtr rfbScreen)
+{
+    const int one = 1;
+    int sock = -1;
+#ifdef LIBVNCSERVER_IPv6
+    struct sockaddr_storage addr;
+#else
+    struct sockaddr_in addr;
+#endif
+    socklen_t addrlen = sizeof(addr);
+    fd_set listen_fds; 
+    int chosen_listen_sock = -1;
+
+    /* Do another select() call to find out which listen socket
+       has an incoming connection pending. We know that at least 
+       one of them has, so this should not block for too long! */
+    FD_ZERO(&listen_fds);  
+    if(rfbScreen->listenSock >= 0) 
+      FD_SET(rfbScreen->listenSock, &listen_fds);
+    if(rfbScreen->listen6Sock >= 0) 
+      FD_SET(rfbScreen->listen6Sock, &listen_fds);
+    if (select(rfbScreen->maxFd+1, &listen_fds, NULL, NULL, NULL) == -1) {
+      rfbLogPerror("rfbProcessNewConnection: error in select");
+      return FALSE;
+    }
+    if (FD_ISSET(rfbScreen->listenSock, &listen_fds)) 
+      chosen_listen_sock = rfbScreen->listenSock;
+    if (FD_ISSET(rfbScreen->listen6Sock, &listen_fds)) 
+      chosen_listen_sock = rfbScreen->listen6Sock;
+
+    if ((sock = accept(chosen_listen_sock,
+		       (struct sockaddr *)&addr, &addrlen)) < 0) {
+      rfbLogPerror("rfbCheckFds: accept");
+      return FALSE;
+    }
+
+    if(!rfbSetNonBlocking(sock)) {
+      closesocket(sock);
+      return FALSE;
+    }
+
+    if (setsockopt(sock, IPPROTO_TCP, TCP_NODELAY,
+		   (char *)&one, sizeof(one)) < 0) {
+      rfbLogPerror("rfbCheckFds: setsockopt");
+      closesocket(sock);
+      return FALSE;
+    }
+
+#ifdef USE_LIBWRAP
+    if(!hosts_ctl("vnc",STRING_UNKNOWN,inet_ntoa(addr.sin_addr),
+		  STRING_UNKNOWN)) {
+      rfbLog("Rejected connection from client %s\n",
+	     inet_ntoa(addr.sin_addr));
+      closesocket(sock);
+      return FALSE;
+    }
+#endif
+
+#ifdef LIBVNCSERVER_IPv6
+    char host[1024];
+    if(getnameinfo((struct sockaddr*)&addr, addrlen, host, sizeof(host), NULL, 0, NI_NUMERICHOST) != 0) {
+      rfbLogPerror("rfbProcessNewConnection: error in getnameinfo");
+    }
+    rfbLog("Got connection from client %s\n", host);
+#else
+    rfbLog("Got connection from client %s\n", inet_ntoa(addr.sin_addr));
+#endif
+
+    rfbNewClient(rfbScreen,sock);
+
+    return TRUE;
+}
+
 
 void
 rfbDisconnectUDPSock(rfbScreenInfoPtr rfbScreen)
@@ -380,6 +478,11 @@ rfbCloseClient(rfbClientPtr cl)
 	  while(cl->screen->maxFd>0
 		&& !FD_ISSET(cl->screen->maxFd,&(cl->screen->allFds)))
 	    cl->screen->maxFd--;
+#ifdef LIBVNCSERVER_WITH_WEBSOCKETS
+	if (cl->sslctx)
+	    rfbssl_destroy(cl);
+	free(cl->wspath);
+#endif
 #ifndef __MINGW32__
 	shutdown(cl->sock,SHUT_RDWR);
 #endif
@@ -445,7 +548,17 @@ rfbReadExactTimeout(rfbClientPtr cl, char* buf, int len, int timeout)
     struct timeval tv;
 
     while (len > 0) {
+#ifdef LIBVNCSERVER_WITH_WEBSOCKETS
+        if (cl->wsctx) {
+            n = webSocketsDecode(cl, buf, len);
+        } else if (cl->sslctx) {
+	    n = rfbssl_read(cl, buf, len);
+	} else {
+            n = read(sock, buf, len);
+        }
+#else
         n = read(sock, buf, len);
+#endif
 
         if (n > 0) {
 
@@ -470,6 +583,12 @@ rfbReadExactTimeout(rfbClientPtr cl, char* buf, int len, int timeout)
                 return n;
             }
 
+#ifdef LIBVNCSERVER_WITH_WEBSOCKETS
+	    if (cl->sslctx) {
+		if (rfbssl_pending(cl))
+		    continue;
+	    }
+#endif
             FD_ZERO(&fds);
             FD_SET(sock, &fds);
             tv.tv_sec = timeout / 1000;
@@ -480,6 +599,7 @@ rfbReadExactTimeout(rfbClientPtr cl, char* buf, int len, int timeout)
                 return n;
             }
             if (n == 0) {
+                rfbErr("ReadExact: select timeout\n");
                 errno = ETIMEDOUT;
                 return -1;
             }
@@ -505,6 +625,82 @@ int rfbReadExact(rfbClientPtr cl,char* buf,int len)
     return(rfbReadExactTimeout(cl,buf,len,rfbMaxClientWait));
 }
 
+/*
+ * PeekExact peeks at an exact number of bytes from a client.  Returns 1 if
+ * those bytes have been read, 0 if the other end has closed, or -1 if an
+ * error occurred (errno is set to ETIMEDOUT if it timed out).
+ */
+
+int
+rfbPeekExactTimeout(rfbClientPtr cl, char* buf, int len, int timeout)
+{
+    int sock = cl->sock;
+    int n;
+    fd_set fds;
+    struct timeval tv;
+
+    while (len > 0) {
+#ifdef LIBVNCSERVER_WITH_WEBSOCKETS
+	if (cl->sslctx)
+	    n = rfbssl_peek(cl, buf, len);
+	else
+#endif
+	    n = recv(sock, buf, len, MSG_PEEK);
+
+        if (n == len) {
+
+            break;
+
+        } else if (n == 0) {
+
+            return 0;
+
+        } else {
+#ifdef WIN32
+	    errno = WSAGetLastError();
+#endif
+	    if (errno == EINTR)
+		continue;
+
+#ifdef LIBVNCSERVER_ENOENT_WORKAROUND
+	    if (errno != ENOENT)
+#endif
+            if (errno != EWOULDBLOCK && errno != EAGAIN) {
+                return n;
+            }
+
+#ifdef LIBVNCSERVER_WITH_WEBSOCKETS
+	    if (cl->sslctx) {
+		if (rfbssl_pending(cl))
+		    continue;
+	    }
+#endif
+            FD_ZERO(&fds);
+            FD_SET(sock, &fds);
+            tv.tv_sec = timeout / 1000;
+            tv.tv_usec = (timeout % 1000) * 1000;
+            n = select(sock+1, &fds, NULL, &fds, &tv);
+            if (n < 0) {
+                rfbLogPerror("PeekExact: select");
+                return n;
+            }
+            if (n == 0) {
+                errno = ETIMEDOUT;
+                return -1;
+            }
+        }
+    }
+#undef DEBUG_READ_EXACT
+#ifdef DEBUG_READ_EXACT
+    rfbLog("PeekExact %d bytes\n",len);
+    for(n=0;n<len;n++)
+	    fprintf(stderr,"%02x ",(unsigned char)buf[n]);
+    fprintf(stderr,"\n");
+#endif
+
+    return 1;
+}
+
 /*
  * WriteExact writes an exact number of bytes to a client.  Returns 1 if
  * those bytes have been written, or -1 if an error occurred (errno is set to
@@ -531,9 +727,25 @@ rfbWriteExact(rfbClientPtr cl,
     fprintf(stderr,"\n");
 #endif
 
+#ifdef LIBVNCSERVER_WITH_WEBSOCKETS
+    if (cl->wsctx) {
+        char *tmp = NULL;
+        if ((len = webSocketsEncode(cl, buf, len, &tmp)) < 0) {
+            rfbErr("WriteExact: WebSockets encode error\n");
+            return -1;
+        }
+        buf = tmp;
+    }
+#endif
+
     LOCK(cl->outputMutex);
     while (len > 0) {
-        n = write(sock, buf, len);
+#ifdef LIBVNCSERVER_WITH_WEBSOCKETS
+        if (cl->sslctx)
+	    n = rfbssl_write(cl, buf, len);
+	else
+#endif
+	    n = write(sock, buf, len);
 
         if (n > 0) {
 
@@ -644,12 +856,128 @@ rfbListenOnTCPPort(int port,
     return sock;
 }
 
+
+int
+rfbListenOnTCP6Port(int port,
+                    const char* iface)
+{
+#ifndef LIBVNCSERVER_IPv6
+    rfbLogPerror("This LibVNCServer does not have IPv6 support");
+    return -1;
+#else
+    int sock;
+    int one = 1;
+    int rv;
+    struct addrinfo hints, *servinfo, *p;
+    char port_str[8];
+
+    snprintf(port_str, 8, "%d", port);
+
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family = AF_INET6;
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_flags = AI_PASSIVE; /* fill in wildcard address if iface == NULL */
+
+    if ((rv = getaddrinfo(iface, port_str, &hints, &servinfo)) != 0) {
+        rfbErr("rfbListenOnTCP6Port error in getaddrinfo: %s\n", gai_strerror(rv));
+        return -1;
+    }
+    
+    /* loop through all the results and bind to the first we can */
+    for(p = servinfo; p != NULL; p = p->ai_next) {
+        if ((sock = socket(p->ai_family, p->ai_socktype, p->ai_protocol)) < 0) {
+            continue;
+        }
+
+#ifdef IPV6_V6ONLY
+	/* we have seperate IPv4 and IPv6 sockets since some OS's do not support dual binding */
+	if (setsockopt(sock, IPPROTO_IPV6, IPV6_V6ONLY, (char *)&one, sizeof(one)) < 0) {
+	  rfbLogPerror("rfbListenOnTCP6Port error in setsockopt IPV6_V6ONLY");
+	  closesocket(sock);
+	  freeaddrinfo(servinfo);
+	  return -1;
+	}
+#endif
+
+	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof(one)) < 0) {
+	  rfbLogPerror("rfbListenOnTCP6Port: error in setsockopt SO_REUSEADDR");
+	  closesocket(sock);
+	  freeaddrinfo(servinfo);
+	  return -1;
+	}
+
+	if (bind(sock, p->ai_addr, p->ai_addrlen) < 0) {
+	  closesocket(sock);
+	  continue;
+	}
+
+        break;
+    }
+
+    if (p == NULL)  {
+        rfbLogPerror("rfbListenOnTCP6Port: error in bind IPv6 socket");
+        freeaddrinfo(servinfo);
+        return -1;
+    }
+
+    /* all done with this structure now */
+    freeaddrinfo(servinfo);
+
+    if (listen(sock, 32) < 0) {
+        rfbLogPerror("rfbListenOnTCP6Port: error in listen on IPv6 socket");
+	closesocket(sock);
+	return -1;
+    }
+
+    return sock;
+#endif
+}
+
+
 int
 rfbConnectToTcpAddr(char *host,
                     int port)
 {
-    struct hostent *hp;
     int sock;
+#ifdef LIBVNCSERVER_IPv6
+    struct addrinfo hints, *servinfo, *p;
+    int rv;
+    char port_str[8];
+
+    snprintf(port_str, 8, "%d", port);
+
+    memset(&hints, 0, sizeof hints);
+    hints.ai_family = AF_UNSPEC;
+    hints.ai_socktype = SOCK_STREAM;
+
+    if ((rv = getaddrinfo(host, port_str, &hints, &servinfo)) != 0) {
+        rfbErr("rfbConnectToTcpAddr: error in getaddrinfo: %s\n", gai_strerror(rv));
+        return -1;
+    }
+
+    /* loop through all the results and connect to the first we can */
+    for(p = servinfo; p != NULL; p = p->ai_next) {
+        if ((sock = socket(p->ai_family, p->ai_socktype, p->ai_protocol)) < 0)
+            continue;
+
+        if (connect(sock, p->ai_addr, p->ai_addrlen) < 0) {
+            closesocket(sock);
+            continue;
+        }
+
+        break;
+    }
+
+    /* all failed */
+    if (p == NULL) {
+        rfbLogPerror("rfbConnectToTcoAddr: failed to connect\n");
+        sock = -1; /* set return value */
+    }
+
+    /* all done with this structure now */
+    freeaddrinfo(servinfo);
+#else
+    struct hostent *hp;
     struct sockaddr_in addr;
 
     memset(&addr, 0, sizeof(addr));
@@ -673,7 +1001,7 @@ rfbConnectToTcpAddr(char *host,
 	closesocket(sock);
 	return -1;
     }
-
+#endif
     return sock;
 }
 
