@@ -1,7 +1,7 @@
 #ifndef _RFB_RFBINT_H
 #define _RFB_RFBINT_H 1
 #ifndef _GENERATED_STDINT_H
-#define _GENERATED_STDINT_H "LibVNCServer 0.9.8.2"
+#define _GENERATED_STDINT_H "LibVNCServer 0.9.9"
 /* generated using a gnu compiler version gcc (Debian 4.4.5-8) 4.4.5 Copyright (C) 2010 Free Software Foundation, Inc. This is free software; see the source for copying conditions. There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. */
 
 #include <stdint.h>
