@@ -1,3 +1,7 @@
+#ifdef __STRICT_ANSI__
+#define _BSD_SOURCE
+#define _POSIX_SOURCE
+#endif
 #include "VNConsole.h"
 #include "vga.h"
 #ifdef LIBVNCSERVER_HAVE_FCNTL_H
